/*
Copyright 2023 ycyxuehan.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package v1alpha1

import (
	"encoding/json"
	"fmt"

	corev1 "k8s.io/api/core/v1"
	rbacv1 "k8s.io/api/rbac/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/types"
)

// EDIT THIS FILE!  THIS IS SCAFFOLDING FOR YOU TO OWN!
// NOTE: json tags are required.  Any new fields you add must have json tags for the fields to be serialized.

// ZGroupSpec defines the desired state of ZGroup
type ZGroupSpec struct {
	// INSERT ADDITIONAL SPEC FIELDS - desired state of cluster
	// Important: Run "make" to regenerate code after modifying this file

	// Foo is an example field of ZGroup. Edit zgroup_types.go to remove/update
	// Foo string `json:"foo,omitempty"`
	Rulers        []rbacv1.PolicyRule      `json:"rulers,omitempty"`
	ClusterRulers []rbacv1.PolicyRule      `json:"clusterRulers,omitempty"`
	Secrets       []corev1.ObjectReference `json:"secrets,omitempty"`
}

// ZGroupStatus defines the observed state of ZGroup
type ZGroupStatus struct {
	// INSERT ADDITIONAL STATUS FIELD - define observed state of cluster
	// Important: Run "make" to regenerate code after modifying this file
}

//+kubebuilder:object:root=true
//+kubebuilder:subresource:status

// ZGroup is the Schema for the zgroups API
type ZGroup struct {
	metav1.TypeMeta   `json:",inline"`
	metav1.ObjectMeta `json:"metadata,omitempty"`

	Spec   ZGroupSpec   `json:"spec,omitempty"`
	Status ZGroupStatus `json:"status,omitempty"`
}

//+kubebuilder:object:root=true

// ZGroupList contains a list of ZGroup
type ZGroupList struct {
	metav1.TypeMeta `json:",inline"`
	metav1.ListMeta `json:"metadata,omitempty"`
	Items           []ZGroup `json:"items"`
}

func init() {
	SchemeBuilder.Register(&ZGroup{}, &ZGroupList{})
}

func (zg *ZGroup) Owner() metav1.OwnerReference {
	bTrue := true
	return metav1.OwnerReference{
		Name:               zg.GetName(),
		APIVersion:         GroupVersion.Version,
		Kind:               "ZGroup",
		UID:                zg.GetUID(),
		Controller:         &bTrue,
		BlockOwnerDeletion: &bTrue,
	}
}

func (zg *ZGroup) ResourceLabels() map[string]string {
	labels := make(map[string]string)
	labels["groups.zelda.io"] = zg.GetName()
	return labels
}

//返回Role
func (zg *ZGroup) Role() *rbacv1.Role {
	if len(zg.Spec.Rulers) == 0 {
		return nil
	}
	role := rbacv1.Role{
		ObjectMeta: metav1.ObjectMeta{
			Name:      zg.Name,
			Namespace: zg.Namespace,
			OwnerReferences: []metav1.OwnerReference{
				zg.Owner(),
			},
			Labels: zg.ResourceLabels(),
		},
		Rules: zg.Spec.Rulers,
	}
	return &role
}

//返回ClusterRole
func (zg *ZGroup) ClusterRole() *rbacv1.ClusterRole {
	if len(zg.Spec.Rulers) == 0 {
		return nil
	}
	role := rbacv1.ClusterRole{
		ObjectMeta: metav1.ObjectMeta{
			Name: zg.Name,
			OwnerReferences: []metav1.OwnerReference{
				zg.Owner(),
			},
			Labels: zg.ResourceLabels(),
		},
		Rules: zg.Spec.ClusterRulers,
	}
	return &role
}

//返回zgroup是否有变更
func (zg *ZGroup) IsSpecChanged() (bool, error) {
	old := ZGroup{}
	latestConfiguration, ok := zg.GetAnnotations()["kubectl.kubernetes.io/last-applied-configuration"]
	if !ok {
		return false, fmt.Errorf("not found annotation")
	}
	err := json.Unmarshal([]byte(latestConfiguration), &old)
	if err != nil { //无法获得旧配置
		return false, err
	}

	oldString, err := json.Marshal(&old.Spec)
	if err != nil {
		return false, err
	}
	newString, err := json.Marshal(&zg.Spec)
	if err != nil {
		return false, err
	}
	if string(oldString) != string(newString) {
		return true, nil
	}
	return false, nil
}

//返回role的namespacedname
func (zg *ZGroup) NamespacedNameRole() types.NamespacedName {
	return types.NamespacedName{
		Namespace: zg.GetNamespace(),
		Name:      zg.GetName(),
	}
}

//返回cluster role的namespaced name
func (zg *ZGroup) NamespacedNameClusterRole() types.NamespacedName {
	return types.NamespacedName{
		Name: zg.GetName(),
	}
}

//ruler和cluster ruler 都为空时，返回true
func (zg *ZGroup) IsEmpty() bool {
	return len(zg.Spec.Rulers) == 0 && len(zg.Spec.ClusterRulers) == 0
}
