/*
Copyright 2023 ycyxuehan.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package v1alpha1

import (
	corev1 "k8s.io/api/core/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/types"
)

// EDIT THIS FILE!  THIS IS SCAFFOLDING FOR YOU TO OWN!
// NOTE: json tags are required.  Any new fields you add must have json tags for the fields to be serialized.
type GitSpec struct {
	Repo string `json:"repo,omitempty"`
	ID   int    `json:"id,omitempty"`
}

type BuildSpec struct {
	Template   corev1.ObjectReference `json:"template,omitempty"`
	Volumes    []corev1.Volume        `json:"volumes,omitempty"`
	Dockerfile string                 `json:"dockerfile,omitempty"`
}

type SubProject struct {
	Name      string    `json:"name,omitempty"`
	Path      string    `json:"path,omitempty"`
	BuildSpec BuildSpec `json:"buildSpec,omitempty"`
	// RunSpec   RunSpec   `json:"runSpec,omitempty"`
}

// ZProjectSpec defines the desired state of ZProject
type ZProjectSpec struct {
	// INSERT ADDITIONAL SPEC FIELDS - desired state of cluster
	// Important: Run "make" to regenerate code after modifying this file

	// Foo is an example field of ZProject. Edit zproject_types.go to remove/update
	//Foo string `json:"foo,omitempty"`
	Git      GitSpec                `json:"git,omitempty"`
	Registry corev1.ObjectReference `json:"registry,omitempty"`
	// ImageName   string       `json:"imageName,omitempty"`
	SubProjects []SubProject `json:"subProjects,omitempty"`
}

// ZProjectStatus defines the observed state of ZProject
type ZProjectStatus struct {
	// INSERT ADDITIONAL STATUS FIELD - define observed state of cluster
	// Important: Run "make" to regenerate code after modifying this file
}

//+kubebuilder:object:root=true
//+kubebuilder:subresource:status

// ZProject is the Schema for the zprojects API
type ZProject struct {
	metav1.TypeMeta   `json:",inline"`
	metav1.ObjectMeta `json:"metadata,omitempty"`

	Spec   ZProjectSpec   `json:"spec,omitempty"`
	Status ZProjectStatus `json:"status,omitempty"`
}

//+kubebuilder:object:root=true

// ZProjectList contains a list of ZProject
type ZProjectList struct {
	metav1.TypeMeta `json:",inline"`
	metav1.ListMeta `json:"metadata,omitempty"`
	Items           []ZProject `json:"items"`
}

func init() {
	SchemeBuilder.Register(&ZProject{}, &ZProjectList{})
}

func (zp *ZProject) SubProject(name string) *SubProject {
	for _, sub := range zp.Spec.SubProjects {
		if sub.Name == name {
			return &sub
		}
	}
	return nil
}

func (zp *ZProject) NamespacedNameRegistry() types.NamespacedName {
	return types.NamespacedName{
		Namespace: zp.Spec.Registry.Namespace,
		Name:      zp.Spec.Registry.Name,
	}
}
