package api

import "github.com/gin-gonic/gin"

type AuthentitionRequest struct {
	ClientID string `json:"clientID,omitempty"`
	Username string `json:"username,omitempty"`
	Password string `json:"password,omitempty"`
	// Namespace		string `json:"namespace,omitempty"`
	Token           string `json:"token,omitempty"`
	Group           string
	KubernetesToken string
}

type AuthentitionResponse struct {
	Token string `json:"token,omitempty"`
	// Username string `json:"username,omitempty"`
	// Group string `json:"group,omitempty"`
	Error   error  `json:"error,omitempty"`
	Message string `json:"message,omitempty"`
	Code    int    `json:"code"`
}

type IdentifyResult struct {
	KubernetesToken string
	Cert            string
}

//验证账号密码的方法，这里想把这一块单独剥离出去，这样不同的auth就更加抽象化
//返回值分别是 kubernetes token，
type IdentifyFunc func(*AuthentitionRequest) (IdentifyResult, error)

type AuthManager interface {
	Login(IdentifyFunc, *AuthentitionRequest) *AuthentitionResponse
	Logout(*AuthentitionRequest) *AuthentitionResponse
	Refresh(*AuthentitionRequest) *AuthentitionResponse
	KubernetesToken(*AuthentitionRequest) (string, error)
	MiddleWare() gin.HandlerFunc
	InitAuthRoute(IdentifyFunc, *gin.RouterGroup)
}
