/*
Copyright 2023 ycyxuehan.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package controller

import (
	"context"

	zeldaiov1alpha1 "github.com/ycyxuehan/zelda/api/v1alpha1"
	corev1 "k8s.io/api/core/v1"
	"k8s.io/apimachinery/pkg/runtime"
	ctrl "sigs.k8s.io/controller-runtime"
	"sigs.k8s.io/controller-runtime/pkg/client"
	"sigs.k8s.io/controller-runtime/pkg/log"
)

// ZRegistryReconciler reconciles a ZRegistry object
type ZRegistryReconciler struct {
	client.Client
	Scheme *runtime.Scheme
}

//+kubebuilder:rbac:groups=zelda.io,resources=zregistries,verbs=get;list;watch;create;update;patch;delete
//+kubebuilder:rbac:groups=zelda.io,resources=zregistries/status,verbs=get;update;patch
//+kubebuilder:rbac:groups=zelda.io,resources=zregistries/finalizers,verbs=update
//+kubebuilder:rbac:groups=*,resources=secrets,verbs=get;list;watch;create;update;patch;delete
//+kubebuilder:rbac:groups=*,resources=secrets/status,verbs=get;update;patch
//+kubebuilder:rbac:groups=*,resources=secrets/finalizers,verbs=update


// Reconcile is part of the main kubernetes reconciliation loop which aims to
// move the current state of the cluster closer to the desired state.
// TODO(user): Modify the Reconcile function to compare the state specified by
// the ZRegistry object against the actual cluster state, and then
// perform operations to make the cluster state reflect the state specified by
// the user.
//
// For more details, check Reconcile and its Result here:
// - https://pkg.go.dev/sigs.k8s.io/controller-runtime@v0.10.0/pkg/reconcile
func (r *ZRegistryReconciler) Reconcile(ctx context.Context, req ctrl.Request) (ctrl.Result, error) {
	_ = log.FromContext(ctx)

	// TODO(user): your logic here
	zregistry := zeldaiov1alpha1.ZRegistry{}
	err := r.Get(ctx, req.NamespacedName, &zregistry)
	if err != nil {
		return ctrl.Result{}, err
	}
	if zregistry.DeletionTimestamp == nil || zregistry.DeletionTimestamp.IsZero() {
		err = r.Process(ctx, &zregistry)
		return ctrl.Result{}, err
	}
	return ctrl.Result{}, nil
}

// SetupWithManager sets up the controller with the Manager.
func (r *ZRegistryReconciler) SetupWithManager(mgr ctrl.Manager) error {
	return ctrl.NewControllerManagedBy(mgr).
		For(&zeldaiov1alpha1.ZRegistry{}).
		Complete(r)
}

//处理zregistry变更
//对于需要账号密码登陆的registry, 自动创建secret并保持一致
func (r *ZRegistryReconciler) Process(ctx context.Context, zregistry *zeldaiov1alpha1.ZRegistry) error {
	secret := &corev1.Secret{}
	err := r.Get(ctx, zregistry.NamespacedNameSecret(), secret)
	changed, _ := zregistry.IsSpecChanged()
	if changed || err != nil {
		secret = zregistry.Secret()
	}
	if err != nil {
		err = r.Create(ctx, secret, &client.CreateOptions{})
		return err
	}
	if changed {
		err = r.Update(ctx, secret, &client.UpdateOptions{})
		return err
	}
	return nil
}
